@@ -25,24 +25,24 @@ a schema extension: https://cyclonedx.org/ext/vulnerability.
 Since CycloneDX schema version 1.4, this has become part of the core schema.
 
 .. note::
-    See the CycloneDX Schema extension definition https://cyclonedx.org/docs/1.4/#type_vulnerabilitiesType
+    See the CycloneDX Schema extension definition https://cyclonedx.org/docs/1.6/#type_vulnerabilitiesType
 """
 
 
 import re
+from collections.abc import Iterable
 from datetime import datetime
 from decimal import Decimal
 from enum import Enum
-from typing import Any, Dict, FrozenSet, Iterable, Optional, Tuple, Type, Union
+from typing import Any, Optional, Union
 
-import serializable
+import py_serializable as serializable
 from sortedcontainers import SortedSet
 
 from .._internal.bom_ref import bom_ref_from_str as _bom_ref_from_str
 from .._internal.compare import ComparableTuple as _ComparableTuple
 from ..exception.model import MutuallyExclusivePropertiesException, NoPropertiesProvidedException
 from ..schema.schema import SchemaVersion1Dot4, SchemaVersion1Dot5, SchemaVersion1Dot6
-from ..serialization import BomRefHelper
 from . import Property, XsUri
 from .bom_ref import BomRef
 from .contact import OrganizationalContact, OrganizationalEntity
@@ -63,7 +63,7 @@ class BomTargetVersionRange:
     `version` and `version_range` are mutually exclusive.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
     """
 
     def __init__(
@@ -126,22 +126,23 @@ class BomTargetVersionRange:
     def status(self, status: Optional[ImpactAnalysisAffectedStatus]) -> None:
         self._status = status
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.version, self.range, self.status
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, BomTargetVersionRange):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, BomTargetVersionRange):
-            return _ComparableTuple((
-                self.version, self.range, self.status
-            )) < _ComparableTuple((
-                other.version, other.range, other.status
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.version, self.range, self.status))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<BomTargetVersionRange version={self.version}, version_range={self.range}, status={self.status}>'
@@ -158,7 +159,7 @@ class BomTarget:
     from a `cyclonedx.parser.BaseParser` implementation.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_vulnerabilityType
     """
 
     def __init__(
@@ -167,7 +168,7 @@ class BomTarget:
         versions: Optional[Iterable[BomTargetVersionRange]] = None,
     ) -> None:
         self.ref = ref
-        self.versions = versions or []  # type:ignore[assignment]
+        self.versions = versions or []
 
     @property
     @serializable.xml_sequence(1)
@@ -197,18 +198,24 @@ class BomTarget:
     def versions(self, versions: Iterable[BomTargetVersionRange]) -> None:
         self._versions = SortedSet(versions)
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.ref,
+            _ComparableTuple(self.versions)
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, BomTarget):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, BomTarget):
-            return self.ref < other.ref
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.ref, tuple(self.versions)))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<BomTarget ref={self.ref}>'
@@ -220,7 +227,7 @@ class VulnerabilityAnalysis:
     Class that models the `analysis` sub-element of the `vulnerabilityType` complex type.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
     """
 
     def __init__(
@@ -229,16 +236,15 @@ class VulnerabilityAnalysis:
         justification: Optional[ImpactAnalysisJustification] = None,
         responses: Optional[Iterable[ImpactAnalysisResponse]] = None,
         detail: Optional[str] = None,
+        first_issued: Optional[datetime] = None,
+        last_updated: Optional[datetime] = None,
     ) -> None:
-        if not state and not justification and not responses and not detail:
-            raise NoPropertiesProvidedException(
-                'At least one of state, justification, responses or detail must be provided for VulnerabilityAnalysis '
-                '- none supplied'
-            )
         self.state = state
         self.justification = justification
-        self.responses = responses or []  # type:ignore[assignment]
+        self.responses = responses or []
         self.detail = detail
+        self.first_issued = first_issued
+        self.last_updated = last_updated
 
     @property
     @serializable.xml_sequence(1)
@@ -306,33 +312,50 @@ class VulnerabilityAnalysis:
     def detail(self, detail: Optional[str]) -> None:
         self._detail = detail
 
-    # @property
-    # @serializable.view(SchemaVersion1Dot5)
-    # @serializable.xml_sequence(5)
-    # def first_issued(self) -> ...:
-    #     ...  # TODO since CDX 1.5
-    #
-    # @first_issued.setter
-    # def first_issued(self, ...) -> None:
-    #     ...  # TODO since CDX 1.5
+    @property
+    @serializable.view(SchemaVersion1Dot5)
+    @serializable.view(SchemaVersion1Dot6)
+    @serializable.type_mapping(serializable.helpers.XsdDateTime)
+    @serializable.xml_sequence(5)
+    def first_issued(self) -> Optional[datetime]:
+        return self._first_issued
 
-    # @property
-    # @serializable.view(SchemaVersion1Dot5)
-    # @serializable.xml_sequence(6)
-    # def last_updated(self) -> ...:
-    #     ...  # TODO since CDX 1.5
-    #
-    # @last_updated.setter
-    # def last_updated(self, ...) -> None:
-    #     ...  # TODO since CDX 1.5
+    @first_issued.setter
+    def first_issued(self, first_issue: Optional[datetime]) -> None:
+        self._first_issued = first_issue
+
+    @property
+    @serializable.view(SchemaVersion1Dot5)
+    @serializable.view(SchemaVersion1Dot6)
+    @serializable.type_mapping(serializable.helpers.XsdDateTime)
+    @serializable.xml_sequence(6)
+    def last_updated(self) -> Optional[datetime]:
+        return self._last_updated
+
+    @last_updated.setter
+    def last_updated(self, last_updated: Optional[datetime]) -> None:
+        self._last_updated = last_updated
+
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.state, self.justification,
+            _ComparableTuple(self.responses),
+            self.detail,
+            self.first_issued, self.last_updated
+        ))
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityAnalysis):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
+    def __lt__(self, other: Any) -> bool:
+        if isinstance(other, VulnerabilityAnalysis):
+            return self.__comparable_tuple() < other.__comparable_tuple()
+        return NotImplemented
+
     def __hash__(self) -> int:
-        return hash((self.state, self.justification, tuple(self.responses), self.detail))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityAnalysis state={self.state}, justification={self.justification}>'
@@ -344,7 +367,7 @@ class VulnerabilityAdvisory:
     Class that models the `advisoryType` complex type.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_advisoryType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_advisoryType
     """
 
     def __init__(
@@ -380,22 +403,23 @@ class VulnerabilityAdvisory:
     def url(self, url: XsUri) -> None:
         self._url = url
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.title, self.url
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityAdvisory):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityAdvisory):
-            return _ComparableTuple((
-                self.title, self.url
-            )) < _ComparableTuple((
-                other.title, other.url
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.title, self.url))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityAdvisory url={self.url}, title={self.title}>'
@@ -409,7 +433,7 @@ class VulnerabilitySource:
     This type is used for multiple purposes in the CycloneDX schema.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilitySourceType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilitySourceType
     """
 
     def __init__(
@@ -417,10 +441,6 @@ class VulnerabilitySource:
         name: Optional[str] = None,
         url: Optional[XsUri] = None,
     ) -> None:
-        if not name and not url:
-            raise NoPropertiesProvidedException(
-                'Either name or url must be provided for a VulnerabilitySource - neither provided'
-            )
         self.name = name
         self.url = url
 
@@ -449,22 +469,23 @@ class VulnerabilitySource:
     def url(self, url: Optional[XsUri]) -> None:
         self._url = url
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.name, self.url
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilitySource):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilitySource):
-            return _ComparableTuple((
-                self.name, self.url
-            )) < _ComparableTuple((
-                other.name, other.url
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.name, self.url))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityAdvisory name={self.name}, url={self.url}>'
@@ -481,62 +502,75 @@ class VulnerabilityReference:
     intelligence.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
+
+    .. note::
+        Properties ``id`` and ``source`` are mandatory.
+
+        History:
+        * In v1.4 JSON scheme, both properties were mandatory
+          https://github.com/CycloneDX/specification/blob/d570ffb8956d796585b9574e57598c42ee9de770/schema/bom-1.4.schema.json#L1455-L1474
+        * In v1.4 XML schema, both properties were optional
+          https://github.com/CycloneDX/specification/blob/d570ffb8956d796585b9574e57598c42ee9de770/schema/bom-1.4.xsd#L1788-L1797
+        * In v1.5 XML schema, both were mandatory
+          https://github.com/CycloneDX/specification/blob/d570ffb8956d796585b9574e57598c42ee9de770/schema/bom-1.5.xsd#L3364-L3374
+
+        Decision:
+        Since CycloneDXCoreWorkingGroup chose JSON schema as the dominant schema, the one that serves as first spec
+        implementation, and since XML schema was "fixed" to work same as JSON schema, we'd consider it canon/spec that
+        both properties were always mandatory.
     """
 
     def __init__(
         self, *,
-        id: Optional[str] = None,
-        source: Optional[VulnerabilitySource] = None,
+        id: str,
+        source: VulnerabilitySource,
     ) -> None:
-        if not id and not source:
-            raise NoPropertiesProvidedException(
-                'Either id or source must be provided for a VulnerabilityReference - neither provided'
-            )
         self.id = id
         self.source = source
 
     @property
     @serializable.xml_sequence(1)
     @serializable.xml_string(serializable.XmlStringSerializationType.NORMALIZED_STRING)
-    def id(self) -> Optional[str]:
+    def id(self) -> str:
         """
         The identifier that uniquely identifies the vulnerability in the associated Source. For example: CVE-2021-39182.
         """
         return self._id
 
     @id.setter
-    def id(self, id: Optional[str]) -> None:
+    def id(self, id: str) -> None:
         self._id = id
 
     @property
     @serializable.xml_sequence(2)
-    def source(self) -> Optional[VulnerabilitySource]:
+    def source(self) -> VulnerabilitySource:
         """
         The source that published the vulnerability.
         """
         return self._source
 
     @source.setter
-    def source(self, source: Optional[VulnerabilitySource]) -> None:
+    def source(self, source: VulnerabilitySource) -> None:
         self._source = source
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.id, self.source
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityReference):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityReference):
-            return _ComparableTuple((
-                self.id, self.source
-            )) < _ComparableTuple((
-                other.id, other.source
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.id, self.source))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityReference id={self.id}, source={self.source}>'
@@ -548,7 +582,7 @@ class VulnerabilityScoreSource(str, Enum):
     Enum object that defines the permissible source types for a Vulnerability's score.
 
     .. note::
-        See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.4/#type_scoreSourceType
+        See the CycloneDX Schema definition: https://cyclonedx.org/docs/1.6/#type_scoreSourceType
 
     .. note::
         No explicit carry-over from the former schema extension:
@@ -623,7 +657,7 @@ class VulnerabilityScoreSource(str, Enum):
 class _VulnerabilityScoreSourceSerializationHelper(serializable.helpers.BaseHelper):
     """  THIS CLASS IS NON-PUBLIC API  """
 
-    __CASES: Dict[Type[serializable.ViewType], FrozenSet[VulnerabilityScoreSource]] = dict()
+    __CASES: dict[type[serializable.ViewType], frozenset[VulnerabilityScoreSource]] = dict()
     __CASES[SchemaVersion1Dot4] = frozenset({
         VulnerabilityScoreSource.CVSS_V2,
         VulnerabilityScoreSource.CVSS_V3,
@@ -638,7 +672,7 @@ class _VulnerabilityScoreSourceSerializationHelper(serializable.helpers.BaseHelp
     __CASES[SchemaVersion1Dot6] = __CASES[SchemaVersion1Dot5]
 
     @classmethod
-    def __normalize(cls, vss: VulnerabilityScoreSource, view: Type[serializable.ViewType]) -> str:
+    def __normalize(cls, vss: VulnerabilityScoreSource, view: type[serializable.ViewType]) -> str:
         return (
             vss
             if vss in cls.__CASES.get(view, ())
@@ -647,14 +681,14 @@ class _VulnerabilityScoreSourceSerializationHelper(serializable.helpers.BaseHelp
 
     @classmethod
     def json_normalize(cls, o: Any, *,
-                       view: Optional[Type[serializable.ViewType]],
+                       view: Optional[type[serializable.ViewType]],
                        **__: Any) -> str:
         assert view is not None
         return cls.__normalize(o, view)
 
     @classmethod
     def xml_normalize(cls, o: Any, *,
-                      view: Optional[Type[serializable.ViewType]],
+                      view: Optional[type[serializable.ViewType]],
                       **__: Any) -> str:
         assert view is not None
         return cls.__normalize(o, view)
@@ -670,7 +704,7 @@ class VulnerabilitySeverity(str, Enum):
     Class that defines the permissible severities for a Vulnerability.
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_severityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_severityType
     """
     NONE = 'none'
     INFO = 'info'  # Only >= 1.4
@@ -681,7 +715,7 @@ class VulnerabilitySeverity(str, Enum):
     UNKNOWN = 'unknown'
 
     @staticmethod
-    def get_from_cvss_scores(scores: Union[Tuple[float, ...], float, None]) -> 'VulnerabilitySeverity':
+    def get_from_cvss_scores(scores: Union[tuple[float, ...], float, None]) -> 'VulnerabilitySeverity':
         """
         Derives the Severity of a Vulnerability from it's declared CVSS scores.
 
@@ -724,7 +758,7 @@ class VulnerabilityRating:
     1.4 - see https://github.com/CycloneDX/specification/blob/master/schema/ext/vulnerability-1.0.xsd.
 
     .. note::
-        See `ratingType` in https://cyclonedx.org/docs/1.4/#ratingType
+        See `ratingType` in https://cyclonedx.org/docs/1.6/xml/#ratingType
 
     .. warning::
         As part of implementing support for CycloneDX schema version 1.4, the three score types defined in the schema
@@ -742,11 +776,6 @@ class VulnerabilityRating:
         vector: Optional[str] = None,
         justification: Optional[str] = None,
     ) -> None:
-        if not source and not score and not severity and not method and not vector and not justification:
-            raise NoPropertiesProvidedException(
-                'At least one property must be provided when creating a VulnerabilityRating - none supplied.'
-            )
-
         self.source = source
         self.score = score
         self.severity = severity
@@ -832,25 +861,30 @@ class VulnerabilityRating:
     def justification(self, justification: Optional[str]) -> None:
         self._justification = justification
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.severity, self.score or 0,
+            self.source, self.method, self.vector,
+            self.justification
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityRating):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityRating):
-            return _ComparableTuple((
-                self.severity, self.score, self.score or 0, self.method, self.vector, self.justification
-            )) < _ComparableTuple((
-                other.severity, other.score, other.score or 0, other.method, other.vector, other.justification
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((self.source, self.score or 0, self.severity, self.method, self.vector, self.justification))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
-        return f'<VulnerabilityRating score={self.score}, source={self.source}>'
+        return f'<VulnerabilityRating severity={self.severity} score={self.score}, ' \
+            f'source={self.source} method={self.method} vector={self.vector}' \
+            f'justification={self.justification}>'
 
 
 @serializable.serializable_class
@@ -862,7 +896,7 @@ class VulnerabilityCredits:
     extension (in XML only).
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/xml/#type_vulnerabilityType
     """
 
     def __init__(
@@ -870,12 +904,8 @@ class VulnerabilityCredits:
         organizations: Optional[Iterable[OrganizationalEntity]] = None,
         individuals: Optional[Iterable[OrganizationalContact]] = None,
     ) -> None:
-        if not organizations and not individuals:
-            raise NoPropertiesProvidedException(
-                'One of `organizations` or `individuals` must be populated - neither were'
-            )
-        self.organizations = organizations or []  # type:ignore[assignment]
-        self.individuals = individuals or []  # type:ignore[assignment]
+        self.organizations = organizations or []
+        self.individuals = individuals or []
 
     @property
     @serializable.xml_array(serializable.XmlArraySerializationType.NESTED, 'organization')
@@ -909,18 +939,24 @@ class VulnerabilityCredits:
     def individuals(self, individuals: Iterable[OrganizationalContact]) -> None:
         self._individuals = SortedSet(individuals)
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            _ComparableTuple(self.organizations),
+            _ComparableTuple(self.individuals)
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, VulnerabilityCredits):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, VulnerabilityCredits):
-            return hash(self) < hash(other)
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((tuple(self.organizations), tuple(self.individuals)))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<VulnerabilityCredits id={id(self)}>'
@@ -935,7 +971,7 @@ class Vulnerability:
     extension (in XML only).
 
     .. note::
-        See the CycloneDX schema: https://cyclonedx.org/docs/1.4/#type_vulnerabilityType
+        See the CycloneDX schema: https://cyclonedx.org/docs/1.6/#type_vulnerabilityType
     """
 
     def __init__(
@@ -963,26 +999,26 @@ class Vulnerability:
         self._bom_ref = _bom_ref_from_str(bom_ref)
         self.id = id
         self.source = source
-        self.references = references or []  # type:ignore[assignment]
-        self.ratings = ratings or []  # type:ignore[assignment]
-        self.cwes = cwes or []  # type:ignore[assignment]
+        self.references = references or []
+        self.ratings = ratings or []
+        self.cwes = cwes or []
         self.description = description
         self.detail = detail
         self.recommendation = recommendation
         self.workaround = workaround
-        self.advisories = advisories or []  # type:ignore[assignment]
+        self.advisories = advisories or []
         self.created = created
         self.published = published
         self.updated = updated
         self.credits = credits
-        self.tools = tools or []  # type:ignore[assignment]
+        self.tools = tools or []
         self.analysis = analysis
-        self.affects = affects or []  # type:ignore[assignment]
-        self.properties = properties or []  # type:ignore[assignment]
+        self.affects = affects or []
+        self.properties = properties or []
 
     @property
     @serializable.json_name('bom-ref')
-    @serializable.type_mapping(BomRefHelper)
+    @serializable.type_mapping(BomRef)
     @serializable.xml_attribute()
     @serializable.xml_name('bom-ref')
     def bom_ref(self) -> BomRef:
@@ -1308,26 +1344,30 @@ class Vulnerability:
     def properties(self, properties: Iterable[Property]) -> None:
         self._properties = SortedSet(properties)
 
+    def __comparable_tuple(self) -> _ComparableTuple:
+        return _ComparableTuple((
+            self.id, self.bom_ref.value,
+            self.source, _ComparableTuple(self.references),
+            _ComparableTuple(self.ratings), _ComparableTuple(self.cwes), self.description,
+            self.detail, self.recommendation, self.workaround, _ComparableTuple(self.advisories),
+            self.created, self.published, self.updated,
+            self.credits, self.tools, self.analysis,
+            _ComparableTuple(self.affects),
+            _ComparableTuple(self.properties)
+        ))
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, Vulnerability):
-            return hash(other) == hash(self)
+            return self.__comparable_tuple() == other.__comparable_tuple()
         return False
 
     def __lt__(self, other: Any) -> bool:
         if isinstance(other, Vulnerability):
-            return _ComparableTuple((
-                self.id, self.description, self.detail, self.source, self.created, self.published
-            )) < _ComparableTuple((
-                other.id, other.description, other.detail, other.source, other.created, other.published
-            ))
+            return self.__comparable_tuple() < other.__comparable_tuple()
         return NotImplemented
 
     def __hash__(self) -> int:
-        return hash((
-            self.id, self.source, tuple(self.references), tuple(self.ratings), tuple(self.cwes), self.description,
-            self.detail, self.recommendation, self.workaround, tuple(self.advisories), self.created, self.published,
-            self.updated, self.credits, self.tools, self.analysis, tuple(self.affects), tuple(self.properties)
-        ))
+        return hash(self.__comparable_tuple())
 
     def __repr__(self) -> str:
         return f'<Vulnerability bom-ref={self.bom_ref.value}, id={self.id}>'
